!function(){ // IIFE declaration

  let old_BM_setup = BattleManager.setup
  BattleManager.setup = function(...setupArgs) {
    let snek_config_installed = $modLoader.knownMods.has('modconfigs')
    if (snek_config_installed) {
      this._randomAfraidEnabled = Snek.ModConfigs.checkConfig('Hard Mode: Random Afraid').index === 1
      this._juiceRegenerationEnabled = Snek.ModConfigs.checkConfig('Hard Mode: Juice Regeneration').index === 1
      this._qteEnabled = Snek.ModConfigs.checkConfig('Hard Mode: Quick Time Event').index === 1
      this._customEnergyUsage = Snek.ModConfigs.checkConfig('Hard Mode: Follow-Up Cost').index ? 5 : 3
    } else { //// VV DEFAULTS
      this._randomAfraidEnabled = false
      this._juiceRegenerationEnabled = false
      this._qteEnabled = false
      this._customEnergyUsage = 5
    }
    old_BM_setup.call(this, ...setupArgs)
  }

  let old_BA_paySkillCost = Game_Actor.prototype.paySkillCost
  Game_Actor.prototype.paySkillCost = function(skill) {
    if ($gameParty.inBattle()) {
      if (Number(skill.meta.EnergyCost) == 3) {
        $gameParty.stressEnergyCount += 3 - BattleManager._customEnergyUsage;
      };
    };
    old_BA_paySkillCost.call(this, skill)
  };

  let old_BA_canPaySkillCost = Game_Actor.prototype.canPaySkillCost
  Game_Actor.prototype.canPaySkillCost = function(skill) {
    if ($gameParty.inBattle()) {
      if (Number(skill.meta.EnergyCost) == 3) {
        return ($gameParty.stressEnergyCount >= BattleManager._customEnergyUsage)
      };
    };
    return old_BA_canPaySkillCost.call(this, skill)
  }
  // Omori Battle System.js overrides
  let old_BM_isBusy = BattleManager.isBusy
  BattleManager.isBusy = function() {
    if (this.isQTEActive) return true;
    return old_BM_isBusy.call(this);
  };

  let old_SB_createStatus = Scene_Battle.prototype.createStatusWindow
  Scene_Battle.prototype.createStatusWindow = function() {
    old_SB_createStatus.call(this)
    this._QTEWindowHitTheMark = new Window_OmoriHitTheMark();
    this.addChild(this._QTEWindowHitTheMark);
  };
  let old_BM_invokeNormalAction = BattleManager.invokeNormalAction
  BattleManager.invokeNormalAction = function(subject, target) {
    old_BM_invokeNormalAction.call(this, subject, target)

    switch (target.$MassiveHit) {
      case 1:
        this._logWindow.push('addText', target._name + " bravely endured the hit!");
        this._logWindow.push('wait');
        break;
      case 2:
        if (target._name != "OMORI") {
          this._logWindow.push('addText', target._name + " got really hurt and became AFRAID.");
        } else {
          this._logWindow.push('addText', target._name + " was not fazed by the pain.");
        }
        this._logWindow.push('wait');
        break;
    }
    target.$MassiveHit = 0;
  };

  // YEP_BattleEngineCore.js overrides
  let old_BM_updateEvent = BattleManager.updateEvent
  BattleManager.updateEvent = function() {
    if (this._phase === "turnEnd" && !this._processingForcedAction) {
      if (this._juiceRegenerationEnabled) {
        let group = $gameParty.aliveMembers();
        for (var i = 0; i < group.length; ++i) {
          group[i].gainMp(1);
        }
      }
      if (this._randomAfraidEnabled) {
        $gameParty.aliveMembers().forEach((member) => {
          if (Math.random() < 0.05) {member.addState(18)}
        });
      }
    }
    return old_BM_updateEvent.call(this)
  }

  let old_BM_processActionSeq = BattleManager.processActionSequence
  BattleManager.processActionSequence = function(actionName, actionArgs) {
    if (actionName === 'PERFORM ACTION' && this._qteEnabled) {
      console.log(this._subject._name + " HIT CHANCE: " + this._subject.hit)

      if (this._subject.hit < 1) {
        if ($gameParty.battleMembers().includes(this._subject) && this._action && this._action.isDamage() && !this._action.hadQTE) {

          let speed = 2; //Base minimum QTE speed was set to 2 from 3
          if (this._subject.isEmotionAffected("happy")) speed = 3; //Speed from HAPPY was set to 3 from 5
          if (this._subject.isEmotionAffected("afraid")) speed = 4; //Speed from AFRAID was set to 4 from 5
          $gameMap._interpreter.startHitTheMarkQTE(speed, undefined, undefined, undefined, this._subject.isEmotionAffected("happy"));
          BattleManager.isQTEActive = true;
          this._actionList.unshift([actionName, actionArgs]);
          return false;
        }
      }
      return this.actionPerformAction();
    }
    return old_BM_processActionSeq.call(this, actionName, actionArgs)
  }

  Yanfly.BEC.Game_Action_apply = function(target) {
      var result = target.result();
      this.subject().clearResult();
      result.clear();
      result.used = this.testApply(target);
      result.missed = (result.used && Math.random() >= this.itemHit(target));

      if ($gameVariables.value(_TDS_.MapQTE.params.QTEResultVariableID) == 1) result.missed = false;

      result.evaded = (!result.missed && Math.random() < this.itemEva(target));
      result.physical = this.isPhysical();
      result.drain = this.isDrain();
      if (result.isHit()) {
          if (this.item().damage.type > 0) {
              result.critical = (Math.random() < this.itemCri(target));
              var value = this.makeDamageValue(target, result.critical);
              this.executeDamage(target, value);
          }
          this.item().effects.forEach(function(effect) {
              this.applyItemEffect(target, effect);
          }, this);
          this.applyItemUserEffect(target);
      }
  }; //why

  let old_GB_performResultEffects = Game_Battler.prototype.performResultEffects
  Game_Battler.prototype.performResultEffects = function() {
      old_GB_performResultEffects.call(this)
      var result = this.result();

      if (result.hpAffected) {
        if (result.hpDamage > 0 && !result.drain) {
          if (this.constructor.name != "Game_Enemy") {
            if (result.hpDamage >= (this.mhp / 3) * 2) {
              if (Math.random() <= 0.5) {
                this.addState(18); this.$MassiveHit = 2;
              } else {
                this.$MassiveHit = 1;
              }

            }
          }

          this.performDamage();
        }
      }
  };

  // Omori Map QTE.js overrides
  Game_Interpreter.prototype.startHitTheMarkQTE = function(speed, index, x, y, canCrit) {
    // Turn ON QTE Switch
    $gameSwitches.setValue(_TDS_.MapQTE.params.activeQTESwitchID, true);
    // Get Window
    var win = SceneManager._scene._QTEWindowHitTheMark;
    // If Window Exists
    if (win) {
      if (index === undefined) { index = Math.randomInt(12); };
      win.open();
      win.activate();
      win.setup(speed, index, canCrit);
      if (x !== undefined) { win.x = x; };
      if (y !== undefined) { win.y = y; };
    };
    this.setWaitMode('QTE');
  };

  Window_OmoriHitTheMark.prototype.initialize = function() {
    // Pointer Move Speed
    this._speed = 1;
    // Movement Direction
    this._direction = 0;
    // Set Target Index
    this._targetIndex = 0;
    // Set Target Width
    this._targetWidth = 8 + (this._speed * 2);
    // Target X
    this._targetX = 0;
    // Super Call
    Window_Base.prototype.initialize.call(this, 14, 0, this.windowWidth(), this.windowHeight());

    // Create Background Sprite
    this.createBackgroundSprite();
    // Create Pointer
    this.createPointer();
    // Center Window
    this.x = (Graphics.width - this.width) / 2;
    this.y = (Graphics.height - this.height) / 2;
    // Refresh Contents
    this.refresh();

    this.deactivate();
    this.openness = 0;
  };

  let old_setup_HTM = Window_OmoriHitTheMark.prototype.setup
  Window_OmoriHitTheMark.prototype.setup = function(speed, index, canCrit) {
    this._canCrit = canCrit
    old_setup_HTM.call(this, speed, index)
  };

  Window_OmoriHitTheMark.prototype.refresh = function() {
    // Clear Contents
    this.contents.clear();
    // Get Bitmap
    var bitmap = ImageManager.loadSystem('QTE_OMORI_ATLAS');
    // console.log(lineSpaceWidth)
    var index = this._targetIndex;
    // Get X Position
    var x = 11 + (index * 17) - index;
    // Set Paint Opacity
    this.contents.paintOpacity = 160;
    bitmap.addLoadListener(() => {
      switch (this._speed) {
        case 2:
          // Set Target Width
          this._targetWidth = 16;
          // Adjust X Position by speed
          x -= 2;
          // Transfer Bitmap to window
          this.contents.blt(bitmap, 209, 0, this._targetWidth, bitmap.height, x, 0);
        break;
        case 3:
          // Set Target Width
          this._targetWidth = 16;
          // Adjust X Position by speed
          x -= 4;
          // Transfer Bitmap to window
          this.contents.blt(bitmap, 220, 0, this._targetWidth, bitmap.height, x, 0);
          break;
        case 4:
          // Adjust X Position by speed
          x -= 4;
          // Set Target Width
          this._targetWidth = 12;
          // Transfer Bitmap to window
          this.contents.blt(bitmap, 225, 0, this._targetWidth, bitmap.height, x, 0);
          break;
        case 5:
          // Adjust X Position by speed
          x -= 6;
          // Set Target Width
          this._targetWidth = 12;
          // Transfer Bitmap to window
          this.contents.blt(bitmap, 220, 0, this._targetWidth, bitmap.height, x, 0);
        break;
      };
      this.contents.paintOpacity = 255;
    })

    // Set Target X
    this._targetX = x;
    // Set Paint Opacity
  };

  Window_OmoriHitTheMark.prototype.finish = function() {
    var padding = 4;
    var startX = this._targetX - padding;
    var endX = this._targetX + this._targetWidth + padding;
    // Check if in Area
    var inArea = (startX < this._pointer.x && this._pointer.x < endX);
    var isCrit = (this._targetX < this._pointer.x) && (this._pointer.x < (this._targetX + this._targetWidth)) && this._canCrit;
    // Turn Off QTE Switch
    $gameSwitches.setValue(_TDS_.MapQTE.params.activeQTESwitchID, false);
    // Set Achieved Segment
    $gameVariables.setValue(_TDS_.MapQTE.params.QTEResultVariableID, inArea ? 1 : 0);
    if (!inArea && BattleManager._action) BattleManager._action.itemHit = () => 0;
    BattleManager._action.itemCri = () => isCrit ? 1 : 0;
    BattleManager._action.hadQTE = true;
    // Close & Deactivate
    this.close(); this.deactivate();
    // Set Visibility to true
    this._pointer.visible = false;
    BattleManager.isQTEActive = false;
  }

  //shop system overrides
  Scene_OmoriItemShop.prototype.onItemListOk = function() {
    var item = this._itemListWindow.item();
    var price = this._itemListWindow.price();
    var buying = $gameTemp._shopData.shopType === 0;
    var maxItems = buying ? $gameParty.maxItems(item) - $gameParty.numItems(item) : $gameParty.numItems(item);
    var canBuy = maxItems > 0
    var hasGold = price <= $gameParty.gold();
    if (buying && (!canBuy || !hasGold)) {
      this.queue(function() {
        var message = '';
        if (!canBuy) { message = $gameTemp._shopData.texts.maxItemMessage}
        if (!hasGold) { message = $gameTemp._shopData.texts.notEnoughMoneyMessage};
        $gameMessage.showLanguageMessage(message);
      }.bind(this))
      this.queue('startFunctionWait', this.waitForMessage.bind(this))
      this.queue(function() {
        this._itemListWindow.activate();
      }.bind(this))
      return;
    };

    // Activate Total Window
    this._totalWindow.open();
    this._totalWindow.setPrice(price, maxItems);
    this._totalWindow.activate();
    // If Buying
    if (buying) {
      // Add Text
      $gameMessage.showLanguageMessage($gameTemp._shopData.texts.onItemListBuyOkMessage);
    } else {
      // Add Text
      $gameMessage.showLanguageMessage($gameTemp._shopData.texts.onItemListSellOkMessage);
    };
  };
}()
